/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

import java.io.IOException;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaPairRDD;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.Function2;
import scala.Tuple2;

/**
 *
 * @author pavel.contos
 */
public class WordCount {

	private static final Logger LOGGER = Logger.getLogger(WordCount.class.getName());

	private static final String APP_NAME = "WordCount";

	private static void process(JavaSparkContext context, String inputFile, String outputFolder) {
		// RDD external dataset example
		// Spark can create distributed datasets from any storage source supported by Hadoop
		// context.textFile(...) method reads the file as a collection of lines
		JavaRDD<String> textFile = context.textFile(inputFile);

		// Transformations and actions (TODO)
		JavaPairRDD<String, Integer> data = textFile
				.flatMap(line -> Arrays.asList(line.split(" ")).iterator())
				.mapToPair(word -> new Tuple2<>(word, 1))
				// When called on a datased of (K, V) pairs, returns a dataset of (K, V) pairs
				// where the values for each key are aggregated using the given reduce function func,
				// which must be of type (V,V) => V. The number of reduce tasks is cofigurable
				// through an optional second argument.
				//.reduceByKey((a, b) -> a + b);
				.reduceByKey(new Function2<Integer, Integer, Integer>() {
					@Override
					public Integer call(Integer a, Integer b) throws Exception {
						return a + b;
					}
				});

		// Action:
		data.saveAsTextFile(outputFolder);
	}

	public static void main(String... args) throws Exception {

		String inputFile, outputFolder;

		switch (args.length) {
			case 2:
				inputFile = args[0];
				outputFolder = args[1];
				break;
			default:
				LOGGER.log(Level.SEVERE, "Invalid use. Two params required: input file, output file.");
				throw new Exception("Invalid use.");
		}

		// Build a SparkConf object that contains information about application.
		SparkConf conf = new SparkConf();
		// Application name to show on the cluster UI
		conf.setAppName(WordCount.APP_NAME);
		// cluster URL (spark://ip_address:7077) or string "local" to run in local mode
		conf.setMaster("local");

		// Context tells Spark how to access a cluster
		JavaSparkContext context = new JavaSparkContext(conf);

		WordCount.process(context, inputFile, outputFolder);

	}
}
